﻿Type=Class
Version=2.71
@EndOfDesignText@
'Class module

'Version: 1.2
'Author: Andreas Huber
'Needs reflection library!

'USAGE:
'------
'You need at least one "strings.xml" - file in the Folder "objects" - "res" - "values"
'Remember to to set this file to read only!!

'To add another language, create a new folder in the "res" directory, named "values-" and the 2 letter abbreviation of the language
'You can find the 2letter abbreviations here: https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes
'Example: "values-de" for german. In this directory, place the localized "strings.xml" - File (Full path: objects/res/values-de/strings.xml"
'Tipp: Services like http://www.getlocalization.com/ can use this xml - files :)

'Sample Layout of the "strings.xml" file:
'<?xml version="1.0" encoding=\"utf-8\"?>
'<resources>
'    <String name="app_name">ahaTranslate</String>
'</resources>

'TIPP:
'Do you want a localized AppName? Go to the Manifest-editor and replace:
'	SetApplicationAttribute(android:Label, "$LABEL$")
'with
'	SetApplicationAttribute(android:label, "@string/app_name")
'Of course, you have to define the app_name in strings.xml as shown above in the Sample Layout

'Info about Translation: http://developer.android.com/guide/topics/resources/localization.html
'http://developer.android.com/guide/topics/resources/string-resource.html
'
' Changes v 1.2:
'---------------
'GetResourceDrawable added

Sub Class_Globals

End Sub
Public Sub Initialize
	
End Sub
'Gets the translation for a specific key
'If no localized String is found, the appropiate String from "values/strings.xml" is is used.
'If the Key doesn't exists, it returns "No translation found!"
Sub GetResourceString(Key As String) As String
    Dim r As Reflector
    Dim package As String
    Dim id As Int
  	
	Try
		package = r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
   		id = r.GetStaticField(package & ".R$string", Key)
    	r.Target = r.GetContext
    	r.Target = r.RunMethod("getResources") 
		Return r.RunMethod2("getText", id, "java.lang.int")
  	Catch 
		Log("No translation for key '" & Key & "' found")
		Return "No translation for key '" & Key & "' found"
	End Try
End Sub
'Gets the translation for a specific key with the use od Parameters.
'string.xml: <string name="Hello">Hi {1} {2}!</string>
'B4A: GetResourceStringWithParams("Hello",Array As String("World",":)")
' => Returnes: Hi World :)!
'If no localized String is found, the appropiate String from "values/strings.xml" is is used.
'If the Key doesn't exists, it returns "No translation found!"
Sub GetResourceStringWithParams(Key As String, Params As List) As String
   	Dim r As Reflector
    Dim package As String
    Dim id As Int
    
	Try
		package = r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
  		id = r.GetStaticField(package & ".R$string", Key)
  		r.Target = r.GetContext
    	r.Target = r.RunMethod("getResources")
   
		Dim tmpText As String 
	
		tmpText = r.RunMethod2("getText", id, "java.lang.int")

		If Params.Size > 0 Then
			For i = 0 To Params.Size - 1
				tmpText = tmpText.Replace ("{" & (i+1) & "}",Params.Get(i))
			Next
		End If
		Return tmpText
	Catch 
		Log("No translation for key '" & Key & "' found")
		Return "No translation for key '" & Key & "' found"
	End Try
End Sub
'Gets an StringArray
'If no localized StringArray is found, the appropiate StringArray from "values/strings.xml" is is used.
'If the Key doesn't exists, it returns "No translation found!"
Sub GetResourceStringArray(Key As String) As String()
    Dim r As Reflector
    Dim package As String
    Dim id As Int
  	
	Try
		package = r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
   		id = r.GetStaticField(package & ".R$array", Key)
    	r.Target = r.GetContext
    	r.Target = r.RunMethod("getResources") 
		Return r.RunMethod2("getStringArray", id, "java.lang.int")
  	Catch 
		Log("No translation for key '" & Key & "' found")
		Return Array As String("No translation for key '" & Key & "' found")
	End Try
End Sub
'Gets localized Plurals. You have to pass the Count. For example:
'count = 1: Returns day
'count =>2: Returns days..
'<link>Quantity Strings|http://developer.android.com/guide/topics/resources/string-resource.html</link>.
'See http://developer.android.com/guide/topics/resources/string-resource.html => Quantity Strings for more information
Sub GetResourceStringPlural(Key As String, Count As Int ) As String
    Dim r As Reflector
    Dim package As String
    Dim id As Int
  	
	Try
		package = r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
   		id = r.GetStaticField(package & ".R$plurals", Key)
    	r.Target = r.GetContext
    	r.Target = r.RunMethod("getResources") 
		Return r.RunMethod3("getQuantityString", id, "java.lang.int",Count, "java.lang.int")
		'Return r.RunMethod2("getQuantityString", id, "java.lang.int")
  	Catch 
		Log("No Plurals for key '" & Key & "' found")
		Return "No Plurals for key '" & Key & "' found"
	End Try
End Sub
'Loads the drawable from the ressources directory (res/drawable) and returnes it as a bitmap. Use res/drawable-de etc for localisation
'Returns Null if not found
Sub GetResourceBitmap(Key As String) As Bitmap 
    Try
		Dim r As Reflector
    	Dim package As String
    	Dim id As Int
		Dim Drawable As BitmapDrawable 
    	package = r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
    	id = r.GetStaticField(package & ".R$drawable", Key)
    	r.Target = r.GetContext
    	r.Target = r.RunMethod("getResources")
    	Drawable = r.RunMethod2("getDrawable", id, "java.lang.int")
		Return Drawable.Bitmap
	Catch
		Return Null
	End Try
End Sub
'Loads the drawable from the ressources directory (res/drawable). Use res/drawable-de etc for localisation
'Returns Null if not found
Sub GetResourceDrawable(Key As String) As  Object  
    Try
		Dim r As Reflector
    	Dim package As String
    	Dim id As Int
		Dim Drawable As BitmapDrawable 
    	package = r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
    	id = r.GetStaticField(package & ".R$drawable", Key)
    	r.Target = r.GetContext
    	r.Target = r.RunMethod("getResources")
    	Drawable = r.RunMethod2("getDrawable", id, "java.lang.int")
		Return Drawable
	Catch
		Return Null
	End Try
End Sub
'Gets the phone language and returns the 2-Letter Code (en for english, de for German..)
'If language cannnot be determined, 'en' is returned
'Returned string is lowercase
'You can find a list here: https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes <link>List|https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes</link>
Sub GetPhoneLanguage As String
	Try
		Dim Sprache As String 
		Dim r As Reflector
    	r.Target = r.RunStaticMethod("java.util.Locale", "getDefault", Null, Null)
    	Sprache = r.RunMethod("getLanguage")
		Return Sprache.ToLowerCase
	Catch
		Log("Phone language not found, returning 'en'")
		Return "en"
	End Try
End Sub
'Gets the phone currency Symbol. If it's not found, $ is returned 
Sub GetPhoneCurrencySymbol As String
	Try
		Dim r As Reflector
		Dim args(1) As Object
		args(0) = r.RunStaticMethod("java.util.Locale", "getDefault", Null, Null)
		Dim Types(1) As String
		Types(0) = "java.util.Locale"	
		r.Target = r.RunStaticMethod("java.util.Currency", "getInstance", args  , Types)
		Return r.RunMethod("getSymbol")
	Catch
		Log("No phone currency symbol found, returning '$'")
		Return "$"
	End Try
End Sub
'Gets the phone Date Settings'  returns "MM/dd/yyyy" on error
Sub GetDeviceDateFormatSettings As String
    Try
		Dim r As Reflector
    	r.Target = r.RunStaticMethod("android.text.format.DateFormat", "getDateFormat", Array As Object(r.GetContext), _
        Array As String("android.content.Context"))
    	Return r.RunMethod("toPattern")
	Catch
		Log("No phone date found, returning 'MM/dd/yyyy'")
		Return "MM/dd/yyyy"
	End Try
End Sub
'Gets a drawable from the Android System resources returned as bitmap, returnes Null if drawable not found
'Here is a list of the possible drawables:
'http://androiddrawableexplorer.appspot.com/ <link>System drawables|http://androiddrawableexplorer.appspot.com/</link>
'You can also look in your SDK-folder: <SDK>\platforms\android-<VERSION>\data\res\
'use the name without the .png extension
Public Sub GetAndroidBitmap(Key As String) As   Bitmap
	Try
		Dim r As Reflector
		Dim Drawable As BitmapDrawable
		r.Target = r.GetContext
		r.Target = r.RunMethod("getResources")
		r.Target = r.RunMethod("getSystem")
		
		Dim ID_Drawable As Int
		ID_Drawable = r.RunMethod4("getIdentifier", Array As Object(Key, "drawable", "android"), _
	                                            Array As String("java.lang.String", "java.lang.String", "java.lang.String"))
		r.Target = r.GetContext
		r.Target = r.RunMethod("getResources")
		Drawable = r.RunMethod2("getDrawable", ID_Drawable, "java.lang.int")
		Return Drawable.Bitmap
	Catch
		Log("No android drawable with key '" & Key & "' found")
		Return Null
	End Try
End Sub
'Gets a drawable from the Android System resources, returnes Null if drawable not found
'Here is a list of the possible drawables:
'http://androiddrawableexplorer.appspot.com/ <link>System drawables|http://androiddrawableexplorer.appspot.com/</link>
'You can also look in your SDK-folder: <SDK>\platforms\android-<VERSION>\data\res\
'use the name without the .png extension
Public Sub GetAndroidDrawable(Key As String) As  Object 
	Try
		Dim r As Reflector
		r.Target = r.GetContext
		r.Target = r.RunMethod("getResources")
		r.Target = r.RunMethod("getSystem")
		
		Dim ID_Drawable As Int
		ID_Drawable = r.RunMethod4("getIdentifier", Array As Object(Key, "drawable", "android"), _
	                                            Array As String("java.lang.String", "java.lang.String", "java.lang.String"))
		r.Target = r.GetContext
		r.Target = r.RunMethod("getResources")
		Return r.RunMethod2("getDrawable", ID_Drawable, "java.lang.int")
	Catch
		Log("No android drawable with key '" & Key & "' found")
		Return Null
	End Try
End Sub
'Translates a whole activity. The Tag information of the controls is used. The tag corresponds to the name in the strings.xml file
'Supported: label, button, checkbox, edittext, radiobutton, togglebutton, spinner
'ATTN: ToggleButtons have to states. ON and OFF. If you use "togglebuttontext" as key in the Tag-Property, you must have 2 Strings in the strings.xml file:
'togglebuttontextON and togglebuttontextOFF!
Sub AutoTranslateActivity(Activity As Activity)
	For i = 0 To Activity.NumberOfViews - 1
		TranslateView(Activity.GetView(i))
	Next
End Sub
Private Sub TranslateView (v As View)
	Try
		'Check for Views inside a Scrollview
		If v Is  ScrollView Then
				Dim Scv As ScrollView
				Scv = v
				For i = 0 To Scv.Panel.NumberOfViews - 1
					TranslateView(Scv.Panel.GetView(i))
				Next
		'Check for Views inside a Panel
		Else If v Is Panel Then
				Dim Pnl As Panel
				Pnl = v
				For i = 0 To Pnl.NumberOfViews - 1
					TranslateView(Pnl.GetView(i))
				Next
		End If

		If v.Tag = "" Then Return 'If no Tag property is set = no translation

		If v Is Spinner Then
			Dim Spn As Spinner
			Spn = v
			Spn.AddAll(GetResourceStringArray(Spn.Tag))

		Else If v Is ToggleButton Then
			Dim Tbtn As ToggleButton
			Tbtn = v
			Tbtn.TextOn  = GetResourceString(Tbtn.Tag & "ON")
			Tbtn.Textoff  = GetResourceString(Tbtn.Tag& "OFF")
		Else If v Is CheckBox Then
			Dim Chk As CheckBox
			Chk = v
			Chk.text = GetResourceString(Chk.Tag)
		Else If v Is RadioButton Then
			Dim Rbtn As RadioButton
			Rbtn = v
			Rbtn.Text  = GetResourceString(Rbtn.Tag)
		Else If v Is EditText Then
			Dim Txt As EditText
			Txt = v
			Txt.Text  = GetResourceString(Txt.Tag)
		Else If v Is Button Then
			Dim Btn As Button
			Btn = v
			Btn.Text  = GetResourceString(Btn.Tag)
		Else If v Is  Label Then
			Dim Lbl As Label
			Lbl = v
			Lbl.Text  = GetResourceString(Lbl.Tag)
		End If
	Catch
		Log("Error translating: " & LastException.Message )
		Return Null
	End Try
End Sub


